//+------------------------------------------------------------------+
//|                                             ROC Smoothed MTF.mq4
//|                                    Copyright  2006, Robert Hill   
//                                  modified by Boxter
//+------------------------------------------------------------------+

#property  copyright "Copyright  2006, Robert Hill"
//---- indicator settings
#property  indicator_separate_window
#property  indicator_buffers 2
#property  indicator_color1  PaleVioletRed
#property  indicator_color2  LimeGreen
#property  indicator_width1  2
#property  indicator_width2  2
#property  indicator_minimum 0
#property  indicator_maximum 1

//
//
//
//
//

extern string TimeFrame           = "Current time frame";
extern int    RPeriod             = 5;
extern bool   UsePercent          = true;
extern int    PriceSmoothing      = 6;
extern double PriceSmoothingPhase = 0;

//
//
//
//
//

double RateOfChange[];
double trend[];
double valUp[];
double valDn[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init(){
//---- drawing settings
   IndicatorBuffers(4);
   SetIndexBuffer(0, valDn); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1, valUp); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2, trend);
   SetIndexBuffer(3, RateOfChange);
   
   //
   //
   //
   //
   //
       
   indicatorFileName   = WindowExpertName();
   returnBars          = TimeFrame=="returnBars";          if (returnBars)     return(0);
   calculateValue      = TimeFrame=="calculateValue";      if (calculateValue) return(0);
   timeFrame           = stringToTimeFrame(TimeFrame);
          
   //
   //
   //
   //
   //
   
   IndicatorShortName(timeFrameToString(timeFrame)+ " ROC2 Smoothed MTF(" + RPeriod + ")");
return(0);
}
  
//+------------------------------------------------------------------+
//| ROC calculation                          |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;
   
   
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { valDn[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame == Period())
   {
   
     for(i=limit; i>=0; i--)
     {
   
         double ROC = iSmooth(Close[i]-Close[i+RPeriod],PriceSmoothing,PriceSmoothingPhase,i);
         
         //
         //
         //
         //
         //
         
         if (UsePercent)
         {
           if (Close[i+RPeriod] != 0)
                RateOfChange[i] = 100 * ROC/Close[i+RPeriod];
         } else RateOfChange[i] = ROC;
         valUp[i] = EMPTY_VALUE;
         valDn[i] = EMPTY_VALUE;
         trend[i] = trend[i+1];
            if (RateOfChange[i] > 0) trend[i] =  1;
            if (RateOfChange[i] < 0) trend[i] = -1;
            if (trend[i] ==  1)      valUp[i] = 1;
            if (trend[i] == -1)      valDn[i] = 1;
      }   
      return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit; i>=0; i--){
      int y = iBarShift(NULL,timeFrame,Time[i]);               
         trend[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RPeriod,UsePercent,PriceSmoothing,PriceSmoothingPhase,2,y);
         valUp[i] = EMPTY_VALUE;
         valDn[i] = EMPTY_VALUE;
            if (trend[i] ==  1) valUp[i] = 1;
            if (trend[i] == -1) valDn[i] = 1;
   }
   return(0);
}
//+------------------------------------------------------------------
//|     Smooth function                                                            
//+------------------------------------------------------------------
double wrk[][10];

#define bsmax  5
#define bsmin  6
#define volty  7
#define vsum   8
#define avolty 9

double iSmooth(double price, double length, double phase, int i, int s=0){
   if (ArrayRange(wrk,0) != Bars) ArrayResize(wrk,Bars);
   
   int r = Bars-i-1; 
   if (r==0) { for(int k=0; k<7; k++) wrk[0][k+s]= price; for(; k<10; k++) wrk[0][k+s]= 0; return(price); }
   
   double len1 = MathMax(MathLog(MathSqrt(0.5*(length-1)))/MathLog(2.0)+2.0,0);
   double pow1 = MathMax(len1-2.0,0.5);
   double del1 = price - wrk[r-1][bsmax+s];
   double del2 = price - wrk[r-1][bsmin+s];
	
   wrk[r][volty+s] = 0;
   if(MathAbs(del1) > MathAbs(del2)) wrk[r][volty+s] = MathAbs(del1); 
   if(MathAbs(del1) < MathAbs(del2)) wrk[r][volty+s] = MathAbs(del2); 
   wrk[r][vsum+s] =	wrk[r-1][vsum+s] + 0.1*(wrk[r][volty+s]-wrk[r-10][volty+s]);
      
   double avgLen = MathMin(MathMax(4.0*length,30),150);
   if (r<avgLen){
      double avg = wrk[r][vsum+s];  
      for (k=1; k<avgLen && (r-k)>=0 ; k++) avg += wrk[r-k][vsum+s];
      avg /= k;
   }
   else avg = (wrk[r-1][avolty+s]*avgLen-wrk[r-toInt(avgLen)][vsum+s]+wrk[r][vsum+s])/avgLen;
                                                          
   wrk[r][avolty+s] = avg;                                           
   if (wrk[r][avolty+s] > 0)double dVolty = wrk[r][volty+s]/wrk[r][avolty+s]; 
   else dVolty = 0;   
	if (dVolty > MathPow(len1,1.0/pow1)) dVolty = MathPow(len1,1.0/pow1);
   if (dVolty < 1)                      dVolty = 1.0;
	        
   double pow2 = MathPow(dVolty, pow1);
   double len2 = MathSqrt(0.5*(length-1))*len1;
   double Kv   = MathPow(len2/(len2+1), MathSqrt(pow2));		
	
   if (del1 > 0) wrk[r][bsmax+s] = price; else wrk[r][bsmax+s] = price - Kv*del1;
   if (del2 < 0) wrk[r][bsmin+s] = price; else wrk[r][bsmin+s] = price - Kv*del2;

   double R     = MathMax(MathMin(phase,100),-100)/100.0 + 1.5;
   double beta  = 0.45*(length-1)/(0.45*(length-1)+2);
   double alpha = MathPow(beta,pow2);

   wrk[r][0+s] = price + alpha*(wrk[r-1][0+s]-price);
   wrk[r][1+s] = (price - wrk[r][0+s])*(1-beta) + beta*wrk[r-1][1+s];
   wrk[r][2+s] = (wrk[r][0+s] + R*wrk[r][1+s]);
   wrk[r][3+s] = (wrk[r][2+s] - wrk[r-1][4+s])*MathPow((1-alpha),2) + MathPow(alpha,2)*wrk[r-1][3+s];
   wrk[r][4+s] = (wrk[r-1][4+s] + wrk[r][3+s]); 

   return(wrk[r][4+s]);
}

int toInt(double value) { return(value); }   

//+------------------------------------------------------------------+
//|  mtf Functions
//+------------------------------------------------------------------+
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

int stringToTimeFrame(string tfs){
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
      if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
   return(Period());
}

string timeFrameToString(int tf){
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
      if (tf==iTfTable[i]) return(sTfTable[i]);
   return("");
}

string StringUpperCase(string str){
   string   s = str;
   for (int length=StringLen(str)-1; length>=0; length--){
      int tchar = StringGetChar(s, length);
      if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
         s = StringSetChar(s, length, tchar - 32);
      else if(tchar > -33 && tchar < 0)
         s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}